@@ -19,34 +19,65 @@
 
      For further information visit http://plib.sourceforge.net
 
-     $Id: pwMacOSX.cxx,v 1.6 2004/03/24 01:30:41 sjbaker Exp $
+     $Id: pwMacOSX.cxx,v 1.7 2004/04/06 23:45:19 sjbaker Exp $
 */
 
-/*
- *  - with Mac OS 8.6 to 9.2:
- *	- must be linked with the following libraries:
- *	InterfaceLib, accessors.o, AppearanceLib and OpenGL (it contains agl)
- *
- *	- ACTIVE_SLEEPTIME	must be defined to 0 for the fastest execution; 
- *			but it will not let other app to get events.
- *
- *  - with Mac OS X: must be linked with the following frameworks:
- *	Carbon, AGL
- *
- */
 
-#include <stdlib.h>
-#include <stdio.h>
 #include "ul.h"
-#include "pw.h"
 
+/* ONLY COMPILE THIS FILE FOR MACINTOSH OR MACOSX SYSTEMS */
+
+/* PLEASE DON'T REMOVE THIS LINE AGAIN!!! */
 #if defined(UL_MACINTOSH) || defined(UL_MAC_OSX)
+/* YES - THAT MEANS YOU! */
 
-#ifdef UL_MAC_OSX
+
+/*
+*
+*  - the fullscreen mode is indeed a "game mode";
+*  - the fullscreen mode does not change the resolution of the screen:
+*		it uses the resolution (width, height and depth) in use before entering pw;
+*  - it is possible to toggle between modes by calling pwCleanup()
+*		and calling again pwInit;
+*  - the "border" parameter in pwInit is not used;
+*  - the "about" menu shows a standard alert box with the current version of plib
+*		and with the url to the plib official site.
+*
+*-----------------------------------------
+*
+*  - with Mac OS 8.6 to 9.2:
+*		- must be linked with the following libraries:
+*			InterfaceLib, accessors.o, AppearanceLib, DrawSprocketLib
+*			and OpenGL (agl is in OpenGL)
+*
+*		- ACTIVE_SLEEPTIME	must be defined to 0 for the fastest execution; 
+*			but it will not let other app to get events.
+*
+*  - with Mac OS X: must be linked with the following frameworks:
+*			Carbon, AGL
+*
+*
+*  - if not using Xcode, the following compiler falgs should be set to avoid errors or warnings
+*		CPPFLAGS = -fpascal-strings -funsigned-char
+*/
+
+/*  version of 2004-04-03
+*   - changes from previous version:
+*		- fullscreen also implemented with mac OS 8.6 and 9;
+*		- key up events are now reported in mac OS 8.6 to 9.2
+*			(these events are disabled by default...)
+*		- multisample added for mac OS X;
+*		- no auto-key repeat event by default (can be set by pwSetAutoRepeatKey(bool on_off) );
+*		- better error handling and use of ulSetError;
+*/
+
+#if defined (UL_MAC_OSX)
 #  define TARGET_API_MAC_CARBON 1
 #  include <Carbon/Carbon.h>
 #  include <AGL/agl.h>
-#elifdef UL_MACINTOSH 
+#elif defined (UL_MACINTOSH)
+//#  define TARGET_API_MAC_CARBON 0
+#  undef ACCESSOR_CALLS_ARE_FUNCTIONS		// already defined by ul.h !!!
 #  define ACCESSOR_CALLS_ARE_FUNCTIONS 1
 #  include <Events.h>
 #  include <MacWindows.h>
@@ -57,29 +88,42 @@
 #  include <Gestalt.h>
 #  include <Appearance.h>
 #  include <Devices.h>
+#  include <CodeFragments.h>
 #  include <agl.h>
+#  include <DrawSprocket.h>
+//#else
+//#  error only for Mac OS (8.6 to OS X) operating system!
 #endif
 
+
+#include <stdlib.h>
+#include <stdio.h>
+#include "pw.h"
+
+
 /* Apple menu: */
 #define mApple	    128
 #define	iAbout	    1
 /* File menu: */
-#  define mFile	    129
-#  define iQuit	    1
-
 #ifdef UL_MAC_OSX
 #  define ACTIVE_SLEEPTIME	0
 #  define INACTIVE_SLEEPTIME  10
+#  define mFile		0
+#  define iQuit		0
 #else   // UL_MACINTOSH
 #  define ACTIVE_SLEEPTIME	1		// 0 is fastest, but does not let other app to catch events.
 									// set to 0 if you want your app only to have CPU time
 #  define INACTIVE_SLEEPTIME  10
+#  define mFile	    129
+#  define iQuit	    1
 #endif
 
 static bool			pwInitialized = false ;
 static int          origin [2]   = {   0,   0 } ;
 static int          size   [2]   = { 640, 480 } ;
 static SInt16		horScrSze, verScrSze;			// screen dimensions
+static bool			full_screen;
+static bool			auto_repeat_key = false ;
 #if TARGET_API_MAC_CARBON
 static bool			pwQuitFlag ;
 #endif
@@ -102,26 +146,34 @@ static pwMousePosFunc *mpCB     = NULL ;
 # define SETPORT(w) SetPort((GrafPtr)w)
 # define GRAFPTR (GrafPtr)
 #endif
-    // generic OpenGL stuff
-static AGLContext   currContext = NULL ;
 
+#ifdef UL_MACINTOSH
+// DrawSprocket stuff:
+static DSpContextReference		dspContext;
+#endif
+
+// generic OpenGL stuff
+static AGLContext   	currContext = NULL ;
+
+static void HandleEvents(void);
 static void Initialize(void);
-static void MakeWindow(int x, int y, int w, int h, char* title);
+static bool MakeWindow(int x, int y, int w, int h, char* title);
 static void CreateContext(int multisample, int num_samples, bool fullscreen);
 static void MakeMenu(void);
-void pwHandleEvents(void);
-//static void DoEvent(EventRecord *event);
 static void handleMenuEvent(long menuResult);
-//static void HandleWindowUpdate(WindowPtr window);
+static void handleKeyEvent(EventRecord* eventPtr, int updown);
+static void handleMouseMoveEvt(EventRecord* eventPtr);
 #if TARGET_API_MAC_CARBON
 static pascal OSErr QuitAppleEventHandler(const AppleEvent *appleEvt, AppleEvent* reply, long refcon);
 #endif
 static void pwAboutBox(void);
 static void defaultExitFunc ();
+#ifdef UL_MACINTOSH
+static bool InitDSP();
+#endif
 
 
-static void CtoPcpy( Str255 pstr, char *cstr );	// dj dans plib ! (?)
-//extern void logOut(const char*, ...);
+static void CtoPcpy( Str255 pstr, char *cstr );	
 
 
 #if TARGET_API_MAC_CARBON
@@ -139,35 +191,29 @@ static void Initialize()
 	BitMap  scrBits;
 
 #if !TARGET_API_MAC_CARBON
-	// MaxApplZone();		// only for 68k code!
-
 	InitGraf(&qd.thePort);
 	InitFonts();
 	FlushEvents(everyEvent, 0);
 	InitWindows();
 	InitMenus();
-	// TEInit();
 	InitDialogs(nil);
-	//qd.randSeed =  TickCount();
 	sleepTime = ACTIVE_SLEEPTIME;
-#endif
-  InitCursor();
-  exitCB = defaultExitFunc;
-#if TARGET_API_MAC_CARBON
-  pwQuitFlag = false;
-  err = AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP(QuitAppleEventHandler), 0, false );
+	SetEventMask(everyEvent); 		// to enable key up events !!!
+#else
+	pwQuitFlag = false;
+	err = AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP(QuitAppleEventHandler), 0, false );
   // this funtion is to handle the "cmd+Q" event. This key event is not send as a keyDown event with carbon!
+	if (err != noErr)
+	{
+		ulSetError(UL_FATAL, "Error in initialization of the system." );
+	}
 #endif
+	InitCursor();
+	exitCB = defaultExitFunc;
 	// get screen size:
 	GetQDGlobalsScreenBits(&scrBits);
 	horScrSze = scrBits.bounds.right -  scrBits.bounds.left;
 	verScrSze = scrBits.bounds.bottom -  scrBits.bounds.top;
-
-	if (err != noErr)
-	{
-		ulSetError(UL_FATAL, "Error in initialization of the system." );
-		exit(3); //ExitToShell(); 
-	}
 }
 
 // Copy a C string into a Pascal string
@@ -178,7 +224,7 @@ static void CtoPcpy(  Str255 pstr, char* cstr )
 	pstr[0] = i - 1;
 }
 
-static void MakeWindow(int x, int y, int w, int h, char* title)
+static bool MakeWindow(int x, int y, int w, int h, char* title)
 {
 	Rect		wRect;
 	Str255		pTitle;
@@ -188,14 +234,18 @@ static void MakeWindow(int x, int y, int w, int h, char* title)
 #if TARGET_API_MAC_CARBON
 	pwWindow = NewCWindow(nil, &wRect, pTitle, true, zoomNoGrow, (WindowPtr) -1, true, 0);
 #else
-	pwWindow = (CWindowPtr) NewCWindow(nil, &wRect, pTitle, true, zoomNoGrow, (WindowPtr) -1, true, 0);
+	if (full_screen)		// use DrawSprocket
+		pwWindow = (CWindowPtr) NewCWindow (nil, &wRect, pTitle, true, plainDBox, (WindowPtr)-1, 0, 0); 
+	else
+		pwWindow = (CWindowPtr) NewCWindow(nil, &wRect, pTitle, true, zoomNoGrow, (WindowPtr) -1, true, 0);
 #endif
-	if (pwWindow == nil)
+	if (pwWindow == NULL)
 	{
-		ulSetError(UL_FATAL, "pw: can not open a window\n");
-		exit(3);
+		ulSetError(UL_WARNING, "pw: can not open a window; will exit");
+		return false;
 	}
 	SETPORT(pwWindow);
+	return true;
 }
 
 static void CreateContext(int multisample, int num_samples, bool fullscreen)
@@ -206,39 +256,43 @@ static void CreateContext(int multisample, int num_samples, bool fullscreen)
 	GLint i = 0;
 	attrib[i++] = AGL_RGBA;
 	attrib[i++] = AGL_DOUBLEBUFFER;
-#if TARGET_API_MAC_CARBON
+#ifdef UL_MAC_OSX
 	if (fullscreen) attrib[i++] = AGL_FULLSCREEN;
 #endif
 	//attrib[i++] = AGL_DEPTH_SIZE;
 	//attrib[i++] = 24;
 	//attrib[i++] = AGL_ALL_RENDERERS;
 	attrib[i++] = AGL_NONE;
-	attrib[i++] = AGL_NONE;
-	attrib[i++] = AGL_NONE;
 	
 	if (multisample)
 	{
-		// attrib[4] = ...
-		// aglPixFmt = aglChoosePixelFormat(NULL, 0, attrib);
-		// if (aglPixFmt == NULL)
-		//{
+#ifdef UL_MAC_OSX
+		i--;
+		attrib[i++] = AGL_SAMPLE_BUFFERS_ARB ; 
+		attrib[i++] = 1 ;
+		attrib[i++] = AGL_SAMPLES_ARB ;
+		attrib[i++] = num_samples;
+		attrib[i++] = AGL_NONE;
+		aglPixFmt = aglChoosePixelFormat(NULL, 0, attrib);
+		if (aglPixFmt == NULL)
+		{
 			multisample = 0;	// retry without
-			ulSetError(UL_WARNING, "pw: multisample not implemented with MacOS 8 & 9\n"); 
-		//}
+			ulSetError(UL_WARNING, "pw: multisample pixel format not found");
+			i -= 5;
+			attrib[i++] = AGL_NONE;
+		}
+#else
+			multisample = 0;	// retry without
+			ulSetError(UL_WARNING, "pw: multisample not implemented with MacOS 8 & 9"); 
+#endif
 	}
 	if (!multisample)
 	{
-		// without multisample:
-		//attrib[4] = AGL_NONE ;
 		aglPixFmt = aglChoosePixelFormat(NULL, 0, attrib);
 	}
 	if (aglPixFmt == NULL)
 	{
-		ulSetError(UL_FATAL, "pw: can not find a pixel format\n");
-		//printf("pw: pixel format  -  Exit\n");
-		//logOut( (const char*) aglErrorString(aglGetError()) );
-  		//logOut(" - can not find a pixel format!\n");
-		exit(2);
+		ulSetError(UL_FATAL, "pw: can not find a pixel format");
 	}
 	
 	// Create an AGL context
@@ -247,10 +301,8 @@ static void CreateContext(int multisample, int num_samples, bool fullscreen)
 	if (currContext == NULL)
 	{
 		ulSetError(UL_FATAL, "pw: can not create an OpenGL context\n");
-  		//logOut("can not create context\n");
-		exit(1);
 	}
-	// swap buffers only during vertical retrace:
+	// swap buffers only every k vertical retrace:
 	//long k = 100;
 	//aglSetInteger(currContext, AGL_SWAP_INTERVAL, &k);
 }
@@ -303,6 +355,11 @@ int pwGetModifiers()
 	return modifiers;
 }
 
+void pwSetAutoRepeatKey( bool on_off )
+{
+	auto_repeat_key = on_off;
+}
+
 static void pwAboutBox()
 {
 	// show a dialog box with info on plib:
@@ -312,12 +369,10 @@ static void pwAboutBox()
 	sprintf(version, "PLIB v %i.%i.%i", PLIB_MAJOR_VERSION, PLIB_MINOR_VERSION, PLIB_TINY_VERSION );
 	CtoPcpy( Pversion, version );
 	StandardAlert ( kAlertPlainAlert,
-   					"\pPLIB v 1.8.1",
+   					Pversion,
    					"\pfor more infos see <http://plib.sourceforge.net>",
    					NULL,
    					&outItemHit  );
-
-	//printf("pwAboutBox appel\n");
 }
 
 static void handleMenuEvent(long menuCmd)
@@ -380,8 +435,7 @@ static int translateKey(int ch, int key)
 		case kHelpCharCode:  		ch = PW_KEY_INSERT;  break;
 		case kFunctionKeyCharCode:
 			// fonction keys:
-			//logOut("key = %X \n", key);
-			switch(key)		// do not work !!!
+			switch(key)		
 			{
 				case 0x7A:  ch = PW_KEY_F1;   break;
 				case 0x78:  ch = PW_KEY_F2;   break;
@@ -403,6 +457,24 @@ static int translateKey(int ch, int key)
 	return ch;
 }
 
+static void handleKeyEvent(EventRecord* eventPtr, int updown)
+{
+	GrafPtr		origPort;
+	
+	if ( ! kbCB ) return;
+	// handle key_events:
+	int ch	= eventPtr->message & charCodeMask ;		// character pressed
+	int key	= (eventPtr->message & keyCodeMask) >> 8 ;	// key pressed
+	ch = translateKey(ch, key);
+	GetPort(&origPort);
+	SETPORT(pwWindow);
+	GlobalToLocal(&eventPtr->where);
+	(*kbCB) ( ch, updown, eventPtr->where.h, eventPtr->where.v ) ;
+	SetPort(origPort);
+	return;
+}
+
+
 static void handleDrag(WindowPtr window, Point mouseloc)
 {
 	Point 		loc;
@@ -447,78 +519,50 @@ static void handleGoAwayBox(WindowPtr window, Point mouseloc)
 	SetPort(origPort);
 }
 
-/*
-static void handleGrow(WindowPtr window, Point mouseloc)
-{
-	Rect	growBounds;
-	long	newSize;
-	GrafPtr		origPort;
-			
-	GetPort(&origPort);
-	
-	SETPORT(window);
-	GetRegionBounds(GetGrayRgn(), &growBounds);
-
-	growBounds.right -= growBounds.left;
-	growBounds.bottom -= growBounds.top;
-	growBounds.left = 20;
-	growBounds.top = 20;
-	
-	newSize = GrowWindow(window, mouseloc, &growBounds);
-	
-	if(newSize)
-	{
-		size[0] = LoWord(newSize);		// save size
-		size[1] = HiWord(newSize);
-		
-		SizeWindow(window, size[1], size[2], false);
-		if ( resizeCB != NULL )  (*resizeCB) ( size[0], size[1] ) ;
-	}
-	
-	SetPort(origPort);
-}
-
-static void handleZoom(WindowPtr window, Point mouseloc, short zoom)
-{
-	GrafPtr		origPort;
-	Rect 		windowBounds;
-	
-	GetPort(&origPort);
-
-	SETPORT(window);
-	
-	if( TrackBox(window, mouseloc, zoom) )
-	{
-		//DoZoomWindow (window, zoom);
-		GetWindowPortBounds(window, &windowBounds); 
-		EraseRect(&windowBounds);	// erase to avoid flicker
-		ZoomWindow (window, zoom, false);
-		
-		size[0] = windowBounds.right - windowBounds.left;
-		size[1] = windowBounds.top - windowBounds.bottom;
-		if ( resizeCB != NULL )  (*resizeCB) ( size[0], size[1] ) ;
-	}
-	
-	SetPort(origPort);
-}
-*/
 static int last_m_h = 0;
 static int last_m_v = 0;
 
-void pwHandleEvents()
+static void handleMouseMoveEvt(EventRecord* eventPtr)
+{
+	GrafPtr		origPort;
+	
+	if ( ! mpCB ) return;
+	
+	int m_h = eventPtr->where.h;
+	int m_v = eventPtr->where.v;
+	if ( (m_v != last_m_v) || (m_h != last_m_h) )
+	{
+		last_m_v = m_v ;    last_m_h = m_h ;		// screen coord.
+		GetPort(&origPort);
+		SETPORT(pwWindow);
+		GlobalToLocal(&eventPtr->where);
+		(*mpCB) ( eventPtr->where.h, eventPtr->where.v ) ;
+		SetPort(origPort);
+	}
+	return;
+}
+
+static void HandleEvents()
 {
   EventRecord	event;
   GrafPtr		origPort;
-  bool			testNextEvent = true;
+//  bool			testNextEvent = true;
 
 #if TARGET_API_MAC_CARBON
   if (pwQuitFlag) (*exitCB)();		// quit!
 #endif
 
-  while ( testNextEvent )
-  {
+//  while ( testNextEvent )
+//  {
 	WaitNextEvent(everyEvent, &event, sleepTime, nil) ;
-	
+#ifdef UL_MACINTOSH
+	if (full_screen)
+	{
+		Boolean dspEventProcessed;
+		DSpProcessEvent(&event, &dspEventProcessed);
+		if (dspEventProcessed) return;
+	}
+#endif
     int updown = PW_UP ;
 
     modifiers = 0;
@@ -540,21 +584,20 @@ void pwHandleEvents()
     			return;
     		}
     		// FALLTHROUGH (not a command)
+			handleKeyEvent(&event, PW_DOWN);
+			return;
     	case autoKey:
+			if ( ! auto_repeat_key )
+			{   // report a mouse move event, if any:
+				handleMouseMoveEvt(&event);
+				return;
+			}
+			// else report a key down event:
     		updown = PW_DOWN ;
     	case keyUp:
-		{
-    		// handle key_events:
-    		int ch	= event.message & charCodeMask ;		// character pressed
-    		int key	= (event.message & keyCodeMask) >> 8 ;	// key pressed
-    		ch = translateKey(ch, key);
-    		GetPort(&origPort);
-    		SETPORT(pwWindow);
-    		GlobalToLocal(&event.where);
-    		if (kbCB) (*kbCB) ( ch, updown, event.where.h, event.where.v ) ;
-    		SetPort(origPort);
-    		return;
-    	}
+			handleKeyEvent(&event, updown);
+			return;
+
     	case mouseDown:	
 		{
 			short	    part;
@@ -566,26 +609,17 @@ void pwHandleEvents()
 				case inMenuBar:
 					handleMenuEvent( MenuSelect(event.where) );
 					return;
-#if !TARGET_API_MAC_CARBON
+		#if !TARGET_API_MAC_CARBON
 				case inSysWindow:
 					SystemClick(&event, window);
-				break;
-#endif
+					break;
+		#endif
 				case inDrag:
 					handleDrag(window, event.where);
 					return;
 				case inGoAway:
 					handleGoAwayBox(window, event.where);
 					return;
-				/*
-				case inGrow:
-					handleGrow(window, event.where);
-					return;
-				case inZoomIn:
-				case inZoomOut:
-					handleZoom(window, event.where, part);
-					return;
-				*/
 				case inContent:
 					if(!window) return;		
 					if ( GRAFPTR pwWindow != FrontWindow() )
@@ -612,7 +646,7 @@ void pwHandleEvents()
     		modifiers = 0;		// reset modifiers with mouse events
     		if (msCB) (*msCB)( button, updown, event.where.h, event.where.v );
     		SetPort(origPort);
-    		break;
+    		return;
 
 		case osEvt:
 		{
@@ -620,6 +654,10 @@ void pwHandleEvents()
 			if (subcode == suspendResumeMessage)   	//	Suspend/resume event	
 				if (resumeFlag & event.message)		//	Resume
 				{
+				#ifdef UL_MACINTOSH
+					if (full_screen) sleepTime = 0;
+					else
+				#endif
 					sleepTime = ACTIVE_SLEEPTIME;
 					return;
 				}
@@ -629,7 +667,14 @@ void pwHandleEvents()
 		}
 		case activateEvt:
 			if (event.modifiers & activeFlag)
-			{ sleepTime = ACTIVE_SLEEPTIME; return; }
+			{
+				#ifdef UL_MACINTOSH
+					if (full_screen) sleepTime = 0;
+					else
+				#endif
+					sleepTime = ACTIVE_SLEEPTIME;
+					return;
+			}
 			else sleepTime = INACTIVE_SLEEPTIME;
 			break;
 		
@@ -637,40 +682,79 @@ void pwHandleEvents()
 		case nullEvent:	// likely an idle event (i.e. no events)
 					// report mouse location, only if mouse has moved:
 					// (there is no mouse move events on Mac!)
-		{
-			int m_h = event.where.h;
-			int m_v = event.where.v;
-			if ( (m_v != last_m_v) || (m_h != last_m_h) )
-			{
-				GetPort(&origPort);
-				SETPORT(pwWindow);
-				GlobalToLocal(&event.where);
-				last_m_v = m_v ;    last_m_h = m_h ;
-				if ( mpCB )  (*mpCB) ( m_h, m_v ) ;
-				SetPort(origPort);
-			}
-	        return;
-		}
+			handleMouseMoveEvt(&event);
+			return;
 		
 		default : /* printf ("event: %i \n", event.what); */ break;
 
     }		// switch(event.what)
-  }		// while ( testNextEvent )
+//  }		// while ( testNextEvent )
 }
 
 
+#ifdef UL_MACINTOSH
+bool InitDSP()
+{
+	DSpContextAttributes	dspAttributes;
+	DisplayIDType			displayID;
+	
+	if ((Ptr) kUnresolvedCFragSymbolAddress == (Ptr) DSpStartup)
+	{
+		ulSetError(UL_WARNING, "DrawSprocket extension not found");
+		return false;
+	}
+	// start DrawSprocket:
+	if (noErr != DSpStartup ())
+	{
+		ulSetError(UL_WARNING, "Unable to start DrawSprocket");
+		return false;
+	}
+	
+	// initialise to zero all fields of DSpContextAttributes:
+	BlockZero( &dspAttributes, sizeof (DSpContextAttributes) );
+	dspAttributes.displayWidth			= horScrSze;
+	dspAttributes.displayHeight			= verScrSze;
+	dspAttributes.colorNeeds			= kDSpColorNeeds_Require;
+	dspAttributes.displayDepthMask		= kDSpDepthMask_16;
+	dspAttributes.displayBestDepth		= 16;
+	dspAttributes.backBufferDepthMask	= kDSpDepthMask_All;	// must be specified even if only fornt buffer is needed
+	dspAttributes.backBufferBestDepth	= 16;
+	dspAttributes.pageCount				= 1;
+	//dspAttributes.contextOptions 		= 0 | kDSpContextOption_DontSyncVBL; // no page flipping and no VBL sync needed
+	// look for a DSp context:
+	dspContext = NULL;
+	OSStatus err = DSpFindBestContext(&dspAttributes, &dspContext);
+	ulSetError(UL_DEBUG, " display width = %li, height = %li,  depth = %li, mask = %lX ",
+			    dspAttributes.displayWidth,
+			    dspAttributes.displayHeight,
+				dspAttributes.displayBestDepth,
+				dspAttributes.displayDepthMask ); 
+	if (err != noErr)
+	{
+		if (err == kDSpContextNotFoundErr)
+			ulSetError(UL_WARNING, "Unable to find a DrawSprocket context");
+		else
+			ulSetError(UL_WARNING, "DSpFindBestContext error");
+		return false;
+	}
+	err = DSpContext_Reserve ( dspContext, &dspAttributes );
+	if (noErr != err)
+	{
+		ulSetError(UL_WARNING, "Unable to set the display!");
+		return false;
+	}
+	// activate DSp context:
+	//DSpContext_FadeGammaOut (NULL, NULL);		// remove for debug
+	DSpContext_SetState (dspContext, kDSpContextState_Active);
+	if ( !MakeWindow( 0, 0, horScrSze, verScrSze, "" ) )  return false;
+	//DSpContext_FadeGammaIn (NULL, NULL);
+	return true;
+}
+#endif
+
 void pwInit ( int multisample, int num_samples )
 {
-	/* Open a full-screen window here please. */
-	// should init DrawSprocket on OS 8.6 to 9 ...
-	//err = DSpStartup ();
-	//if (err != noErr)  logOut("DSpStartup error.\n");
-#if TARGET_API_MAC_CARBON
 	pwInit ( 0,0,-1,-1, multisample, "", 0, num_samples ) ;
-#else
-	ulSetError(UL_FATAL, "fullscreen not implemented with os < X \n");
-	exit(3);
-#endif
 }
 
 
@@ -683,24 +767,53 @@ void pwInit ( int x, int y, int w, int h, int multisample,
 	return;
   }
   
-  bool full_screen = ( (w<0) || (h<0) ) ? true : false ;
+  full_screen = ( (w<0) || (h<0) ) ? true : false ;
   Initialize();
+  pwInitialized = true;
   pwWindow = NULL;
   
+  // Initialize OpenGL stuff (format, context, ...):
   CreateContext(multisample, num_samples, full_screen);
   
-#if TARGET_API_MAC_CARBON
-  if (full_screen) {
-	// Initialize OpenGL stuff (format, context, ...):
+  if (full_screen)
+  {
+	sleepTime = 0;
+#ifdef UL_MACINTOSH
+	// initialisation of fullscreen mode with DrawSprocket
+	dspContext = NULL;
+	if ( ! InitDSP() )
+	{
+		pwCleanup();	// releases GL contexts, windows, ...
+		exit(4);
+	}
+	// attach OpenGl to the screen:
+	aglSetDrawable(currContext, GetWindowPort (GRAFPTR pwWindow));
+	/*
+	ulSetError(UL_DEBUG, "aglSetDrawable error: %s",
+					(const char*) aglErrorString(aglGetError()) );
+	*/
+	// activate our GL context
+	aglSetCurrentContext(currContext);
+#else
 	aglSetCurrentContext(currContext);
 	aglSetFullScreen (currContext, 0, 0, 0, 0);
-  } else
 #endif
+	// initialise origin and size of the window (for pwGetSize)
+	origin[0] = 0;
+	origin[1] = 0;
+	size[0] = horScrSze;
+	size[1] = verScrSze;
+  }
+  else	// full_screen
   {
-  	MakeWindow(x,y,w,h, title);
-  	// Attach the context to the window
+	  if ( ! MakeWindow(x,y,w,h, title) )
+	  {
+		  pwCleanup();	// releases GL contexts, windows, ...
+		  exit(4);
+	  }
+	  // Attach the context to the window
   	aglSetDrawable(currContext, GetWindowPort (GRAFPTR pwWindow));
-#if !TARGET_API_MAC_CARBON
+#ifdef UL_MACINTOSH
   	{	// because aglSetDrawable is slow... (not needed with OSX)
 		EventRecord event;
 		WaitNextEvent (everyEvent, &event, 2, NULL);
@@ -710,8 +823,12 @@ void pwInit ( int x, int y, int w, int h, int multisample,
 
 	// create a default menu bar (with the "quit" command):
 	MakeMenu();
+	// initialise origin and size of the window (for pwGetSize)
+	origin[0] = x;
+	origin[1] = y;
+	size[0] = w;
+	size[1] = h;
   }
-  pwInitialized = true ; 
 }
 
 
@@ -722,7 +839,7 @@ void pwGetSize ( int *w, int *h )
 }
 
 //special cursors:
-static Cursor q_curs =
+static Cursor q_curs =		// question mark cursor
 {
 	{0x0380, 0x07c0, 0x0c60, 0x1830, 0x1830, 0x0030, 0x0060, 0x00c0,
 	 0x0180, 0x0180, 0x0180, 0, 0, 0x0180, 0x0180, 0},	// data
@@ -730,7 +847,7 @@ static Cursor q_curs =
 	 0x01e0, 0x01c0, 0x01c0, 0x01c0, 0, 0x01c0, 0x01c0, 0x01c0},	// mask
 	{ 14, 7 } 	// hotspot
 };
-static Cursor c_curs =
+static Cursor c_curs =		// circle cursor
 {
 	{0, 0x03e0, 0x0c18, 0x1004, 0x2002, 0x2002, 0x4001, 0x4001,
 	 0x4001, 0x4001, 0x4001, 0x2002, 0x2002, 0x1004, 0x0c18, 0x03e0},	// data
@@ -738,7 +855,7 @@ static Cursor c_curs =
 	 0x6003, 0x6003, 0x6003, 0x3006, 0x3006, 0x1c1c, 0x0ff8, 0x03e0},	// mask
 	{ 8, 8 } 	// hotspot
 };
-static Cursor r_curs =
+static Cursor r_curs =		// right arrow cursor
 {
 	{0, 0x0008, 0x0018, 0x0038, 0x0078, 0x00f8, 0x01f8, 0x03f8,
 	 0x07f8, 0x00f8, 0x00d8, 0x0188, 0x0180, 0x0300, 0x0300, 0},	 // data
@@ -746,7 +863,7 @@ static Cursor r_curs =
 	 0x0ffc, 0x0ffc, 0x01fc, 0x03dc, 0x03cc, 0x0780, 0x0780, 0x0700}, // mask
 	{ 1, 12 } 	// hotspot
 };
-static Cursor x_curs =
+static Cursor x_curs =		// circle + cross cursor
 {
 	{0, 0x03e0, 0x0c98, 0x1084, 0x2082, 0x2082, 0x4081, 0x4081,
 	 0x7fff, 0x4081, 0x4081, 0x2082, 0x2082, 0x1084, 0x0c98, 0x03e0},	// data
@@ -765,7 +882,7 @@ void pwSetCursor ( int c )
 		case PW_CURSOR_NONE   : HideCursor(); return;
 		case PW_CURSOR_LEFT   : csr = kThemeArrowCursor; break;
 		case PW_CURSOR_RIGHT  : csrPtr = &r_curs; break;
-		case PW_CURSOR_AIM    : csrPtr = &x_curs; break; // ???
+		case PW_CURSOR_AIM    : csrPtr = &x_curs; break;
 		//case PW_CURSOR_CARET  : csr = kThemeIBeamCursor; break; // ???
 		case PW_CURSOR_WAIT   : csr = kThemeWatchCursor; break;
 		case PW_CURSOR_CROSS  : csr = kThemeCrossCursor; break;
@@ -788,6 +905,12 @@ void pwSetSize ( int w, int h )
 		ulSetError(UL_WARNING, "call to pwSetSize without call to pwInit");
 		return;
 	}
+	if ( full_screen ) 
+	{
+		ulSetError(UL_WARNING, "call to pwSetSize in full screen mode");
+		return;
+	}
+	if ( ( w == size[0] ) && ( h == size[1] ) ) return;
 	
 	GrafPtr		origPort;
 	
@@ -818,6 +941,12 @@ void pwSetOrigin ( int x, int y )
 		ulSetError(UL_WARNING, "call to pwSetOrigin without call to pwInit");
 		return;
 	}
+	if ( full_screen ) 
+	{
+		ulSetError(UL_WARNING, "call to pwSetOrigin in full screen mode");
+		return;
+	}
+	if ( ( x == origin[0] ) && ( y == origin[1] ) ) return;
 	
 	if ( (x + size[0]) < 10)   x = 10 - size[0];
 	if ( x > (horScrSze-10) )  x = horScrSze-10;
@@ -840,6 +969,11 @@ void pwSetSizeOrigin ( int x, int y, int w, int h )
 		ulSetError(UL_WARNING, "call to pwSetSizeOrigin without call to pwInit");
 		return;
 	}
+	if ( full_screen ) 
+	{
+		ulSetError(UL_WARNING, "call to pwSetSizeOrigin in full screen mode");
+		return;
+	}
 	
 	pwSetOrigin( x, y );
 	pwSetSize( w, h );
@@ -850,12 +984,12 @@ void pwSwapBuffers ()
 {
 	if (!pwInitialized)
 	{
-		ulSetError(UL_FATAL, "call to pwSwapBuffers without call to pwInit");
-		exit(1);
+		ulSetError(UL_WARNING, "call to pwSwapBuffers without call to pwInit");
+		return;
 	}
 	// glFlush () ;
 	aglSwapBuffers ( currContext ) ;
-	pwHandleEvents () ;
+	HandleEvents () ;
 }
 
 
@@ -874,8 +1008,24 @@ void pwCleanup ()
 		aglDestroyContext ( currContext );
 	}
 	if (pwWindow != NULL)   DisposeWindow(GRAFPTR pwWindow);
-}
-
+	
+#ifdef UL_MACINTOSH
+	if (full_screen)
+	{
+		if (dspContext != NULL)
+		{
+			//DSpContext_FadeGammaOut (NULL, NULL);	// remove for debug
+			DSpContext_SetState (dspContext, kDSpContextState_Inactive);
+			//DSpContext_FadeGammaIn (NULL, NULL);	
 
+			DSpContext_Release (dspContext);
+		}
+			
+		DSpShutdown ();
+	}
 #endif
 
+	pwInitialized = false;
+}
+
+#endif
