@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: slMODPrivate.h,v 1.4 2001/04/12 03:37:36 deca Exp $
+     $Id: slMODPrivate.h,v 1.7 2002/11/30 00:41:49 sjbaker Exp $
 */
 
 
@@ -29,7 +29,7 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 
-#if defined( WIN32 ) && !defined( __CYGWIN32__ )
+#ifdef UL_MSVC
 #include <windows.h>
 #include <io.h>
 #else
