@@ -18,7 +18,7 @@
 
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssg.cxx,v 1.65 2003/12/12 17:23:51 wolfram_kuss Exp $
+     $Id: ssg.cxx,v 1.70 2004/10/29 22:24:08 wolfram_kuss Exp $
 */
 
 
@@ -63,6 +63,9 @@ sgVec4 _ssgColourWhite = { 1.0f, 1.0f, 1.0f, 1.0f } ;
 sgVec3 _ssgNormalUp    = { 0.0f, 0.0f, 1.0f } ;
 sgVec2 _ssgTexCoord00  = { 0.0f, 0.0f } ;
 short  _ssgIndex0      = 0;
+bool   _ssgBackFaceCollisions = false ;
+
+void ssgSetBackFaceCollisions ( bool b ) { _ssgBackFaceCollisions = b ; }
 
 static ssgLight _ssgLights [ 8 ] ;
 static int      _ssgFrameCounter = 0 ;
@@ -142,10 +145,10 @@ void ssgInit ()
   ssgAddModelFormat ( ".atg"  , ssgLoadATG  , ssgSaveATG ) ;
   ssgAddModelFormat ( ".qhi"  , NULL        , ssgSaveQHI ) ;
   ssgAddModelFormat ( ".wrl",   ssgLoadVRML1, ssgSaveVRML1 ) ;
-  ssgAddModelFormat ( ".iv", ssgLoadIV      , NULL ) ;
-  ssgAddModelFormat ( ".asc",   NULL        , ssgSaveASC ) ;
+  ssgAddModelFormat ( ".iv",    ssgLoadIV   , ssgSaveIV  ) ;
+  ssgAddModelFormat ( ".asc",   ssgLoadASC  , ssgSaveASC ) ;
 //ssgAddModelFormat ( ".pov",   NULL        , ssgSavePOV ) ;
-	
+  
 
 #ifdef SSG_LOAD_MDL_SUPPORTED
   ssgAddModelFormat ( ".mdl",   ssgLoadMDL  , NULL       ) ;
@@ -354,7 +357,8 @@ static ssgBase *createBranch ()            { return new ssgBranch           ; }
 //static ssgBase *createBaseTransform ()     { return new ssgBaseTransform    ; }
 static ssgBase *createTransform ()         { return new ssgTransform        ; }
 static ssgBase *createTexTrans ()          { return new ssgTexTrans         ; }
-static ssgBase *createAxisTransform()      { return new ssgAxisTransform; }
+static ssgBase *createAnimTransform()      { return new ssgAnimTransform    ; }
+static ssgBase *createAxisTransform()      { return new ssgAxisTransform    ; }
 static ssgBase *createSelector ()          { return new ssgSelector         ; }
 static ssgBase *createRangeSelector ()     { return new ssgRangeSelector    ; }
 static ssgBase *createTimedSelector ()     { return new ssgTimedSelector    ; }
@@ -397,9 +401,10 @@ static struct {
   { ssgTypeBranch ()            , createBranch            },
 //{ ssgTypeBaseTransform ()     , createBaseTransform     },
   { ssgTypeTransform ()         , createTransform         },
+  { ssgTypeAnimTransform ()     , createAnimTransform     },
   { ssgTypeTexTrans ()          , createTexTrans          },
-	{ ssgTypeAxisTransform ()     , createAxisTransform     },
-	{ ssgTypeSelector ()          , createSelector          },
+  { ssgTypeAxisTransform ()     , createAxisTransform     },
+  { ssgTypeSelector ()          , createSelector          },
   { ssgTypeRangeSelector ()     , createRangeSelector     },
   { ssgTypeTimedSelector ()     , createTimedSelector     },
   { ssgTypeTweenController ()   , createTweenController   },
@@ -431,7 +436,7 @@ void ssgRegisterType ( int type, ssgBase * ( *func ) () )
   if ( type == 0 || func == NULL ) 
   {
     ulSetError ( UL_WARNING, "ssgRegisterType: Bad arguments (type %#x, func %p).",
-		 type, func ) ;
+     type, func ) ;
     return ;
   }
 
@@ -466,8 +471,8 @@ ssgBase *ssgCreateOfType ( int type )
     ulSetError ( UL_WARNING, "ssgCreateOfType: Got null object for type %#x.", type ) ;
   else if ( obj -> getType () != type )
     ulSetError ( UL_WARNING,
-		 "ssgCreateOfType: Created object has wrong type %#x (%s), expected %#x.",
-		 obj -> getType (), obj -> getTypeName (), type ) ;
+     "ssgCreateOfType: Created object has wrong type %#x (%s), expected %#x.",
+     obj -> getType (), obj -> getTypeName (), type ) ;
 
   return obj ;
 }
