@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgCutout.cxx,v 1.8 2001/06/16 16:27:37 stromberg Exp $
+     $Id: ssgCutout.cxx,v 1.11 2002/09/02 06:05:48 sjbaker Exp $
 */
 
 
@@ -132,6 +132,10 @@ void ssgCutout::hot ( sgVec3 s, sgMat4 m, int test_needed )
   ssgBranch::hot ( s, m, test_needed ) ;
 }
 
+void ssgCutout::los ( sgVec3 s, sgMat4 m, int test_needed )
+{
+  ssgBranch::los ( s, m, test_needed ) ;
+}
 
 
 void ssgCutout::isect ( sgSphere *s, sgMat4 m, int test_needed )
