@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,16 +13,16 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgLoadFLT.cxx,v 1.25 2001/06/16 21:03:27 sjbaker Exp $
+     $Id: ssgLoadFLT.cxx,v 1.39 2003/04/17 13:10:12 stromberg Exp $
 */
 
 //
-//  $Id: ssgLoadFLT.cxx,v 1.25 2001/06/16 21:03:27 sjbaker Exp $
+//  $Id: ssgLoadFLT.cxx,v 1.39 2003/04/17 13:10:12 stromberg Exp $
 //
 //
 //    OpenFlight loader for SSG
@@ -90,26 +90,21 @@
 /*#define NO_COLORS*/
 
 /* Try to figure out how to do mmap. */
-#ifdef _WIN32
+#ifdef UL_WIN32
 # define USE_WIN32_MMAP
-#elif defined(__sgi) || defined(__GNUC__) /* OK? Any Unix dialect should work. */
+#else
 # include <unistd.h>
 # ifdef _POSIX_MAPPED_FILES
 #  define USE_POSIX_MMAP
 # endif
 #endif
 
-#define USE_ALLOCA
-
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
 #include <stdio.h>
-#ifdef __sgi
+#ifdef UL_IRIX
 # include <sys/endian.h>
-# ifdef USE_ALLOCA
-#  include <alloca.h>
-# endif
 #endif
 #ifdef USE_POSIX_MMAP
 # include <time.h>
@@ -117,17 +112,8 @@
 # include <sys/stat.h>
 # include <sys/mman.h>
 #endif
-#ifdef _MSC_VER
+#ifdef UL_MSVC
 # include <io.h>
-# ifdef USE_ALLOCA
-#  include <malloc.h>
-# endif
-#endif
-
-#ifdef __MWERKS__
-#  ifdef USE_ALLOCA
-#    include <alloca.h>
-#  endif
 #endif
 
 
@@ -156,7 +142,7 @@
 
 typedef unsigned char ubyte;
 
-#ifdef _WIN32
+#ifdef UL_WIN32
 typedef unsigned short ushort;
 typedef unsigned int uint;
 #endif
@@ -171,7 +157,7 @@ typedef unsigned int uint;
 /* XXX what about PDP_ENDIAN? */
 
 /* Help! Is this correct? */
-#if !defined(BYTE_ORDER) && defined(_MSC_VER) && !defined(NOT_INTEL_BYTE_ORDER)
+#if (!defined(BYTE_ORDER) && defined(UL_WIN32) && !defined(NOT_INTEL_BYTE_ORDER))
 # define LITTLE_ENDIAN  1234
 # define BIG_ENDIAN     4321
 # define BYTE_ORDER     LITTLE_ENDIAN
@@ -181,6 +167,7 @@ typedef unsigned int uint;
 
 #if !defined(BYTE_ORDER) || BYTE_ORDER != BIG_ENDIAN
 
+#if !defined(BYTE_ORDER)
 static void _swab16(const void *src, void *dst, int n)
 {
    ushort *s = (ushort *)src;
@@ -191,6 +178,7 @@ static void _swab16(const void *src, void *dst, int n)
 	      ((t & 0x00ffU) << 8));
    }
 }
+#endif
 
 static void _swab32(const void *src, void *dst, int n)
 {
@@ -334,7 +322,7 @@ struct snode {
 
 typedef int (*sfunc)(const void *key1, const void *key2);
 
-struct snode *splay(struct snode *t, const void *key, sfunc comp)
+static struct snode *splay(struct snode *t, const void *key, sfunc comp)
 {
    struct snode N, *l, *r, *y;
      
@@ -598,8 +586,13 @@ struct fltState {
 struct fltNodeAttr {   
 
    /* allocated using new/delete for convenience */
-   fltNodeAttr() { memset(this, 0, sizeof(*this)); }
-   ~fltNodeAttr() { if (name) /*free(name);*/ delete name; }
+   fltNodeAttr() {
+      memset(this, 0, sizeof(*this)); 
+   }
+   ~fltNodeAttr() { 
+      if (name) delete name; 
+      if (mask_words) delete mask_words; 
+   }
 
    /* properies that are not applied immediately */
 
@@ -613,6 +606,12 @@ struct fltNodeAttr {
    sgVec2 range;
    sgVec3 center;
 
+   /* switch record info */
+   int num_masks;
+   int num_words;
+   uint *mask_words;
+   int current_mask;
+
 };
 
 static int Inited = 0;
@@ -701,7 +700,7 @@ static ssgSimpleState *ConstructState(StateInfo *key)
    if (StateCache->data == (void *)-1) {
       ssgSimpleState *s;
 
-      /*printf("new state --\n");*/
+      /*ulSetError(UL_DEBUG, "new state --");*/
 
       s = new ssgSimpleState;
 
@@ -710,13 +709,13 @@ static ssgSimpleState *ConstructState(StateInfo *key)
       }
       else {
 	 s->disable(GL_CULL_FACE);
-	 /*printf("  two-sided\n");*/
+	 /*ulSetError(UL_DEBUG, "  two-sided");*/
       }
 
       if (key->tr) {
 	 s->setTranslucent();
 	 s->enable(GL_BLEND); /* XXX what about multisampling? */
-	 /*printf("  transparent\n");*/
+	 /*ulSetError(UL_DEBUG, "  transparent");*/
       }
       else {
 	 s->setOpaque();
@@ -727,37 +726,37 @@ static ssgSimpleState *ConstructState(StateInfo *key)
       if (key->mtl) {
 	 float *m = key->mtl;
 	 sgVec4 c;
-	 /*printf("%.2f %.2f %.2f  %.2f %.2f %.2f  %.2f %.2f %.2f  %.2f %.2f %.2f  %.2f %.2f  %d\n",
+	 /*ulSetError(UL_DEBUG, "%.2f %.2f %.2f  %.2f %.2f %.2f  %.2f %.2f %.2f  %.2f %.2f %.2f  %.2f %.2f  %d",
 	   m[0], m[1], m[2], m[3], m[4], m[5], m[6], m[7], m[8], m[9], m[10], m[11], m[12], m[13], key->cm);*/
 	 if (key->cm) {
 	    s->enable(GL_COLOR_MATERIAL);
 	    s->setColourMaterial(GL_AMBIENT_AND_DIFFUSE);
-	    //printf("  color material\n");
+	    //ulSetError(UL_DEBUG, "  color material");
 	 }
 	 else {
 	    s->disable(GL_COLOR_MATERIAL);
 	    sgSetVec4(c, m[0], m[1], m[2], key->alpha); s->setMaterial(GL_AMBIENT, c);
 	    sgSetVec4(c, m[3], m[4], m[5], key->alpha); s->setMaterial(GL_DIFFUSE, c);
-	    //printf("  ambient   %.2f %.2f %.2f\n", m[0], m[1], m[2]);
-	    //printf("  diffuse   %.2f %.2f %.2f\n", m[3], m[4], m[5]);
+	    //ulSetError(UL_DEBUG, "  ambient   %.2f %.2f %.2f", m[0], m[1], m[2]);
+	    //ulSetError(UL_DEBUG, "  diffuse   %.2f %.2f %.2f", m[3], m[4], m[5]);
 	 }
 	 sgSetVec4(c, m[6], m[7], m[8], key->alpha); s->setMaterial(GL_SPECULAR, c);
 	 sgSetVec4(c, m[9], m[10], m[11], key->alpha); s->setMaterial(GL_EMISSION, c);
-	 //printf("  specular  %.2f %.2f %.2f\n", m[6], m[7], m[8]);
-	 //printf("  emission  %.2f %.2f %.2f\n", m[9], m[10], m[11]);
-	 //printf("  alpha %.2f\n", key->alpha);
+	 //ulSetError(UL_DEBUG, "  specular  %.2f %.2f %.2f", m[6], m[7], m[8]);
+	 //ulSetError(UL_DEBUG, "  emission  %.2f %.2f %.2f", m[9], m[10], m[11]);
+	 //ulSetError(UL_DEBUG, "  alpha %.2f", key->alpha);
 	 s->setShininess(m[12]);
 	 s->enable(GL_LIGHTING);
       }
       else {
 	 s->disable(GL_LIGHTING);
-	 /*printf("  no lighting\n");*/
+	 /*ulSetError(UL_DEBUG, "  no lighting");*/
       }
 
       if (key->tex) {	 
 	 s->enable(GL_TEXTURE_2D);
 	 s->setTexture(key->tex);
-	 /*printf("  texture %s\n", key->tex->getFilename());*/
+	 /*ulSetError(UL_DEBUG, "  texture %s", key->tex->getFilename());*/
       }
       else {
 	 s->disable(GL_TEXTURE_2D);
@@ -845,13 +844,8 @@ static ssgEntity *Build(fltState *state)
     */
    qsort(arr, num, sizeof(fltTriangle), tricmp);
 
-#ifdef USE_ALLOCA
-   int *index = (int *)alloca(sizeof(index[0]) * state->vnum);
-   int *vertex = (int *)alloca(sizeof(vertex[0]) * 32768);
-#else
-   int index[state->vnum];
-   int vertex[32768];
-#endif
+   int *index = new int [ state->vnum ];
+   int *vertex = new int [ 32768 ];
    
    for (i = 0; i < num; ) {
 
@@ -1027,7 +1021,7 @@ static ssgEntity *Build(fltState *state)
 
       geom->setState(arr[i].state);
       
-      /*printf("build: [%d..%d] %02x %d indices, %d vertices\n",
+      /*ulSetError(UL_DEBUG, "build: [%d..%d] %02x %d indices, %d vertices",
 	     i, j - 1, bind,
 	     3*(j - i), n);*/
       
@@ -1071,6 +1065,9 @@ static ssgEntity *Build(fltState *state)
       i = j;
    }
 
+   delete [] vertex;
+   delete [] index;
+
    if (((leaf != 0) + (bboard1 != 0) + (bboard2 != 0)) > 1 && !grp) //lint !e514 Lint warning "unusual use of a boolean"
       grp = new ssgBranch;
 
@@ -1114,14 +1111,13 @@ static void AddTri(fltState *state, int v0, int v1, int v2)
 
 static void Triangulate(int *w, int n, fltState *state)
 {
-#ifdef USE_ALLOCA
-   int *tris = (int *) alloca(sizeof(int) * 3 * (n - 2));
-#else
-   int tris[3 * (n - 2)];
-#endif
+   int buf[3 * (16 - 2)];
+   int *tris = (n > 16) ? new int [ 3 * (n - 2) ] : buf;
    int num_tris = _ssgTriangulate(state->coord, w, n, tris);
    for (int i = 0; i < num_tris; i++)
       AddTri(state, tris[3*i + 0], tris[3*i + 1], tris[3*i + 2]);
+   if (tris != buf)
+     delete [] tris;
 }
 
 static int ObsoleteFlag;
@@ -1133,6 +1129,7 @@ static void Obsolete(int op)
       ulSetError(UL_WARNING, "[flt] This file is probably rather old (obsolete opcodes ignored).");
       ObsoleteFlag = 1;
    }
+   //printf("op %d obsolete\n", op);
 }
 
 static void NotImplemented(int op)
@@ -1141,6 +1138,7 @@ static void NotImplemented(int op)
       ulSetError(UL_WARNING, "[flt] This file contains opcodes that are not implemented.");
       NotImplementedFlag = 1;
    }
+   //printf("op %d not implemented\n", op);
 }
 
 static void ReportBadChunk(const ubyte *ptr, const char *name)
@@ -1293,7 +1291,7 @@ static int GeomChunks(ubyte *ptr0, ubyte *end, fltState *state, ssgEntity **node
 	    if (index != -1) {
 	       state->texs = t = splay(state->texs, (void *)index, ptrcmp);
 	       if (t == 0 || t->key != (void *)index) {
-		  /*fprintf(stderr, "[flt] undefined texture %d\n", index);*/
+		  /*ulSetError(UL_DEBUG, "[flt] undefined texture %d", index);*/
 		  t = 0;
 	       }
 	    }
@@ -1330,7 +1328,7 @@ static int GeomChunks(ubyte *ptr0, ubyte *end, fltState *state, ssgEntity **node
 	       int lmode = len >= 49 ? ptr[48] : 3;
 	       int trans = get16u(ptr + 40);
 
-	       //printf("light mode %d\n", lmode);
+	       //ulSetError(UL_DEBUG, "light mode %d", lmode);
 
 	       /*lmode = 0;*/
 	    
@@ -1427,10 +1425,10 @@ static int GeomChunks(ubyte *ptr0, ubyte *end, fltState *state, ssgEntity **node
 
 	       /*
 		 if (tri.bind & BIND_COLOR) 
-		   printf("use vertex colors\n");
+		   ulSetError(UL_DEBUG, "use vertex colors");
 		 if (tri.bind & BIND_FLAT_COLOR)
-		   printf("face color %.2f %.2f %.2f\n", 
-			  tri.color[0], tri.color[1], tri.color[2]);
+		   ulSetError(UL_DEBUG, "face color %.2f %.2f %.2f", 
+			      tri.color[0], tri.color[1], tri.color[2]);
 	        */
 
 	       /* billboard */
@@ -1758,7 +1756,7 @@ static ssgEntity *PostClean(ssgEntity *node, fltNodeAttr *attr)
 
    /* set limits on animated nodes */
    if (node && node->isA(ssgTypeTimedSelector())) {
-      ssgTimedSelector *sw = (ssgTimedSelector *)node;
+      ssgTimedSelector *sw = (ssgTimedSelector *) node;
       if (sw->getNumKids() > 1) {
 	 sw->setDuration(30.0f);
 	 sw->setLimits(0, sw->getNumKids() - 1);			  
@@ -1783,9 +1781,9 @@ static ssgEntity *PostClean(ssgEntity *node, fltNodeAttr *attr)
 	    grp->addKid(scs);
 	    sgCopyMat4(mat, attr->mat);
 #if 0
-	    printf("replicating %d times\n", attr->replicate);
+	    ulSetError(UL_DEBUG, "replicating %d times", attr->replicate);
 	    for (i = 0; i < 4; i++)
-	       printf("  %6.2f %6.2f %6.2f %6.2f\n", mat[i][0], mat[i][1], mat[i][2], mat[i][3]);
+	       ulSetError(UL_DEBUG, "  %6.2f %6.2f %6.2f %6.2f", mat[i][0], mat[i][1], mat[i][2], mat[i][3]);
 #endif
 	    for (i = 0; i < attr->replicate; ++i) {
 	       sgPostMultMat4(mat, attr->mat);
@@ -1795,11 +1793,11 @@ static ssgEntity *PostClean(ssgEntity *node, fltNodeAttr *attr)
 	       grp->addKid(scs);
 	    }
 	    node = grp; /* there's no point in cleaning this construction */
-	    /*printf("[flt] replicated %d times\n", attr->replicate);*/
+	    /*ulSetError(UL_DEBUG, "[flt] replicated %d times", attr->replicate);*/
 	 }
 	 else {
 	    node = scs; // PostClean(scs, 0);
-	    /*printf("[flt] added transformation\n");*/
+	    /*ulSetError(UL_DEBUG, "[flt] added transformation");*/
 	 }
       }
       
@@ -1830,7 +1828,31 @@ static ssgEntity *PostClean(ssgEntity *node, fltNodeAttr *attr)
 	       lod->addKid(node);
 	       node = lod;
 	    }
-	 }	 
+	 }
+      }
+
+      /* setup switch records */
+      if (attr->num_masks > 0) {
+	ssgTimedSelector *sw = new ssgTimedSelector;
+	int n = ((ssgBranch *) node) -> getNumKids();
+	if (n > 32 * attr->num_words) n = 32 * attr->num_words;
+	for (int i = 0; i < attr->num_masks; i++) {
+	  ssgBranch *br = new ssgBranch;
+	  for (int j = 0; j < n; j++) {
+	    if (attr->mask_words[j / 32] & (1u << (j % 32)))
+	      br -> addKid(((ssgBranch *) node) -> getKid(j));
+	  }
+	  sw -> addKid(br);
+	}
+	if (node->getRef() == 0) {
+	  ((ssgBranch *) node) -> removeAllKids();
+	  delete node;
+	}
+	sw->setMode(SSG_ANIM_SHUTTLE);
+	sw->setDuration(30.0f);
+	sw->setLimits(0, attr->num_masks - 1);	
+	sw->control(SSG_ANIM_START);
+	node = sw;
       }
    }
 
@@ -1843,10 +1865,11 @@ static ssgEntity *PostClean(ssgEntity *node, fltNodeAttr *attr)
 }
 
 /* link nodes while removing crap (post traversal) */
-static void PostLink(ssgEntity **stack, fltNodeAttr **attr)
+static void PostLink(ssgEntity **stack, fltNodeAttr **attr, 
+		     int instance, fltState *state)
 {
 #if 0
-   fprintf(stderr, "PostLink: %s %s <-- %s %s\n",
+   ulSetError(UL_DEBUG, "PostLink: %s %s <-- %s %s",
            stack[0] ? pfGetTypeName(stack[0]) : "0", stack[0] ? pfGetNodeName(stack[0]) : "0",
            stack[1] ? pfGetTypeName(stack[1]) : "0", stack[1] ? pfGetNodeName(stack[1]) : "0");
 #endif
@@ -1859,14 +1882,28 @@ static void PostLink(ssgEntity **stack, fltNodeAttr **attr)
       return;
    }
    assert(!stack[1]->isA(0xDeadBeef));
+
+   stack[1] = PostClean(stack[1], attr[1]);
+   
+   if (stack[1] && instance >= 0) {
+       state->refs = sinsert(state->refs, (void *)instance, 0, ptrcmp);
+       if (state->refs->data != (void *)-1) {
+	   ulSetError(UL_WARNING, "[flt] Instance %d redefined.", instance);
+	   ssgDeRefDelete((ssgEntity *) state->refs->data);
+       }
+       state->refs->data = stack[1];
+       stack[1]->ref();
+   }
    
    if (stack[0] == 0) {
-      stack[0] = stack[1]; /* delay further processing */
-      attr[0] = attr[1];
+      stack[0] = stack[1];
+      if (attr[0]) {
+	  delete attr[0];
+	  attr[0] = 0;
+      }
    }
    else {
       assert(!stack[0]->isA(0xDeadBeef));
-      stack[1] = PostClean(stack[1], attr[1]);
       if (stack[1]) {
          if (stack[0]->isAKindOf(ssgTypeBranch())) {
 	    ((ssgBranch *)stack[0])->addKid(stack[1]);
@@ -1879,6 +1916,7 @@ static void PostLink(ssgEntity **stack, fltNodeAttr **attr)
          }
       }
    }
+
    stack[1] = 0;
    attr[1] = 0;
 }
@@ -2006,13 +2044,16 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
 {
    ssgEntity *stack[MAXDEPTH + 1];
    fltNodeAttr *attr[MAXDEPTH + 1];
+   int instance[MAXDEPTH + 1];
    int sp, op, len, k;
 
    stack[0] = new ssgBranch;
    stack[0]->setName("reserved");
    attr[0] = 0;
+   instance[0] = -1;
    stack[1] = 0;
    attr[1] = 0;
+   instance[1] = -1;
    sp = 1;
 
    for (;;) {
@@ -2025,6 +2066,12 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
       len = get16u(ptr + 2);
       if (len < 4 || (len & 3) != 0 || ptr + len > end)
          break;
+
+#if 0
+      for (int i = 0; i < sp; i++)
+	  putchar('\t');
+      printf("op %d\n", op);
+#endif
       
       switch (op) {
 
@@ -2032,7 +2079,8 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
          if (stack[sp] && !stack[sp]->isAKindOf(ssgTypeBranch())) {
             /* shouldn't happen */
             ulSetError(UL_DEBUG, "[flt] Objects are not allowed to contain other objects or groups.");
-            PostLink(stack + sp - 1, attr + sp - 1);
+            PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+	    instance[sp] = -1;
          }
 	 if (sp >= MAXDEPTH) {
 	    ulSetError(UL_WARNING, "[flt] Stack overflow.");
@@ -2041,6 +2089,7 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
 	    sp++;
 	    stack[sp] = 0;
 	    attr[sp] = 0;
+	    instance[sp] = -1;
 	 }
 	 ptr += len;
          break;
@@ -2049,8 +2098,8 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
          if (sp == 1) 
             ulSetError(UL_WARNING, "[flt] Stack underflow.");
          else {
-            PostLink(stack + sp - 1, attr + sp - 1);
-            sp--;            
+            PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+            sp--;
          }
          ptr += len;
          break;
@@ -2084,7 +2133,8 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
 
 	 int flags = get32i(ptr + 12);
 	 int trans = get16u(ptr + 18);
-         PostLink(stack + sp - 1, attr + sp - 1);
+         PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+	 instance[sp] = -1;
          ptr += len;
          ptr += AttrChunks(ptr, end, &attr[sp]);
 	 state->parent_name = (char *)ptr + 4; /* OK?? */
@@ -2098,7 +2148,8 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
       case 5: /* Face */
          /* polygons are not allowed to be outside objects, but this rule is not always respected */
 	 //ulSetError(UL_DEBUG, "[flt] Implicit object.");
-         PostLink(stack + sp - 1, attr + sp - 1);
+         PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+	 instance[sp] = -1;
          ptr += GeomChunks(ptr, end, state, &stack[sp], 0, 0);
          break;
 
@@ -2108,20 +2159,33 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
       case 96: /* Switch */
       case 98: /* Clip Region */
 
-         PostLink(stack + sp - 1, attr + sp - 1);
+         PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+	 instance[sp] = -1;
 
 	 switch (op) {
-	 case 14: 
-	    stack[sp] = new ssgTransform; 
-	    break;
-#if 1
-	 case 96: {
-	    ssgTimedSelector *sw = new ssgTimedSelector; 
-	    stack[sp] = sw;
+
+	 case 2: { /* Group */
+#if 0 /* group animation - seems wrong... */
+	    int flags = (len >= 20) ? get32i(ptr + 16) : 0;
+	    if ((flags & 3) != 0) {
+	       ssgTimedSelector *sw = new ssgTimedSelector;
+	       sw->setMode((flags & 2) ? SSG_ANIM_SWING : SSG_ANIM_SHUTTLE);
+	       stack[sp] = sw;
+	    }
+	    else 
+#endif
+            {
+	       stack[sp] = new ssgBranch;
+	    }
 	    break;
 	 }
-#endif
-	 case 73: {
+
+	 case 14: { /* Degree of Freedom */
+	    stack[sp] = new ssgTransform; 
+	    break;
+	 }
+
+	 case 73: { /* Level of Detail */
 	    fltNodeAttr *a;
 	    double v[3];
 	    attr[sp] = a = new fltNodeAttr;
@@ -2130,33 +2194,63 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
 	    sgSetVec2(a->range, (float)v[0], (float)v[1]);
 	    get64v(ptr + 40, v, 3);
 	    sgSetVec3(a->center, (float)v[0], (float)v[1], (float)v[2]);
-	    /* break; */
-	 }
-	 default: //lint !e616
             stack[sp] = new ssgBranch;
+	    break;
+	 }
+
+	 case 96: { /* Switch */
+	    int num_words = (len > 28) ? get32i(ptr + 20) : -1;
+	    int num_masks = (len > 28) ? get32i(ptr + 24) : -1;
+	    if (num_words <= 0 || num_masks <= 0 || len < 28 + 4 * num_words * num_masks) {
+ 	       ulSetError(UL_DEBUG, "[flt] Incomplete switch record.");
+	    }
+	    else {
+ 	       fltNodeAttr *a = new fltNodeAttr;
+	       a->current_mask = get32i(ptr + 16);
+	       a->num_masks = num_masks;
+	       a->num_words = num_words;
+	       a->mask_words = new uint [ num_masks * num_words ];
+	       get32v(ptr + 28, a->mask_words, num_masks * num_words);
+	       attr[sp] = a;
+	    }
+	    stack[sp] = new ssgBranch;
+	    break;
+	 }
+
+	 default: { /* Others (i.e. 98 Clip Region) */
+	    stack[sp] = new ssgBranch;
+	    break;
+	 }
+
          }
+
 	 if (ptr[4])
 	    stack[sp]->setName((char *)ptr + 4);
+
          ptr += len;
          ptr += AttrChunks(ptr, end, &attr[sp]);
+
          break;
 
       case 61: /* Instance Reference */
          k = get16u(ptr + 6);
-         /*fprintf(stderr, "[flt] instance reference %d\n", k);*/
-         PostLink(stack + sp - 1, attr + sp - 1);	 
-         if (state->refs) {
-	    state->refs = splay(state->refs, (void *)k, ptrcmp);
-	    if (state->refs->key == (void *)k)
-	       stack[sp] = (ssgEntity *)state->refs->data;
+	 if (k != instance[sp]) { // current instance inserted by default
+	     PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+	     instance[sp] = -1;
+	     if (state->refs) {
+		 state->refs = splay(state->refs, (void *)k, ptrcmp);
+		 if (state->refs->key == (void *)k) {
+		     stack[sp] = (ssgEntity *)state->refs->data;
+		 }
+	     }
 	 }
-         ptr += len;
+	 ptr += len;
          ptr += AttrChunks(ptr, end, &attr[sp]);
          break;
 
       case 62: /* Instance Definition */
          k = get16u(ptr + 6);
-         /*fprintf(stderr, "[flt] instance definition %d\n", k);*/
+#if 0	 
          if (stack[sp]) {
 	    state->refs = sinsert(state->refs, (void *)k, 0, ptrcmp);
 	    if (state->refs->data == (void *)-1) {
@@ -2164,12 +2258,20 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
 	       stack[sp]->ref();
 	    }
 	 }
+	 else 
+	 {
+	     ulSetError(UL_DEBUG, "[flt] yeah, but the node is empty!");
+	 }
+#else	 
+	 instance[sp] = k;
+#endif
          ptr += len;
          break;
 
       case 63: /* External Reference */
-         /*fprintf(stderr, "external reference %s\n", ptr + 4);*/
-         PostLink(stack + sp - 1, attr + sp - 1);
+         /*ulSetError(UL_DEBUG, "external reference %s", ptr + 4);*/
+         PostLink(stack + sp - 1, attr + sp - 1, instance[sp], state);
+	 instance[sp] = -1;
          if (!NoExternals) {
             char *file = (char *)ptr + 4, *p;
 	    if ((p = strrchr(file, '/')))
@@ -2193,6 +2295,7 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
       case 101: /* Light Source Bead */
       case 122: /* Push Attribute */
       case 123: /* Pop Attribute */
+	//printf("op %d ignored\n", op);
 	 ptr += len;
 	 break;
 
@@ -2212,7 +2315,7 @@ static ssgEntity *HierChunks(ubyte *ptr, ubyte *end, fltState *state)
 
    /* pop stack (expected one iteration but may be more for incomplete databases) */
    while (sp-- > 0)
-      PostLink(stack + sp, attr + sp);
+      PostLink(stack + sp, attr + sp, -1, 0);
    
    if (stack[0])
       stack[0] = PostClean(stack[0], attr[0]);
@@ -2232,7 +2335,7 @@ static int VertexTable(ubyte *ptr0, ubyte *end, fltState *state)
    len = get16u(ptr + 2);
 
    size = get32i(ptr + 4);
-   /*printf("vertex table header len %d, total size %d\n", len, size);*/
+   /*ulSetError(UL_DEBUG, "vertex table header len %d, total size %d", len, size);*/
    num = (size - len)/40; /* max # of vertices */
    if (num <= 0 || state->vtab) {
       if (state->vtab)
@@ -2280,7 +2383,7 @@ static int VertexTable(ubyte *ptr0, ubyte *end, fltState *state)
       if (!(flags & 4)) {
 	 if ((flags & 8) && p + 4 <= ptr + len) {
 	    UNPACK_ABGR(state->color[i], p);
-	    //printf("packed ABGR: %d %d %d %d\n", p[0], p[1], p[2], p[3]);
+	    //ulSetError(UL_DEBUG, "packed ABGR: %d %d %d %d", p[0], p[1], p[2], p[3]);
 	    bind |= BIND_COLOR;
 	 }
 	 else if (state->revision > 1400) {
@@ -2296,7 +2399,7 @@ static int VertexTable(ubyte *ptr0, ubyte *end, fltState *state)
 	    int index = color / 128;
 	    int intensity = color % 128;
 	    if (color >= 0 && state->ctab && index < state->cnum) {
-	       //printf("colour %d: index %d, intensity %d\n", color, index, intensity);
+	       //ulSetError(UL_DEBUG, "colour %d: index %d, intensity %d", color, index, intensity);
 	       UNPACK_ABGR2(state->color[i], state->ctab[index], intensity);
 	       bind |= BIND_COLOR;
 	    }
@@ -2304,7 +2407,7 @@ static int VertexTable(ubyte *ptr0, ubyte *end, fltState *state)
       }
       /*
        if (bind & BIND_COLOR)
-	 printf("vertex #%d color %.2f %.2f %.2f\n", i,
+	 ulSetError(UL_DEBUG, "vertex #%d color %.2f %.2f %.2f", i,
 		state->color[i][0],
 		state->color[i][1],
 		state->color[i][2]);
@@ -2312,7 +2415,7 @@ static int VertexTable(ubyte *ptr0, ubyte *end, fltState *state)
       state->bind[i] = bind;
       ptr += len;      
    }
-   /*printf("got %d vertices\n", i);*/
+   /*ulSetError(UL_DEBUG, "got %d vertices", i);*/
 
    state->vnum = i;
 
@@ -2357,9 +2460,9 @@ static int TableChunks(ubyte *ptr0, ubyte *end, fltState *state)
 #if 0
 	    int i;
 	    for (i = 0; i < state->cnum; i++)
-	       printf("%d %02x %02x %02x %02x\n", i, 
-		      state->ctab[i][0], state->ctab[i][1], 
-		      state->ctab[i][2], state->ctab[i][3]);
+	       ulSetError(UL_DEBUG, "%d %02x %02x %02x %02x", i, 
+		                    state->ctab[i][0], state->ctab[i][1], 
+		                    state->ctab[i][2], state->ctab[i][3]);
 #endif
 	 }
 	 else {
@@ -2477,6 +2580,8 @@ static int TableChunks(ubyte *ptr0, ubyte *end, fltState *state)
       case 93: /* Sound Palette */
       case 97: /* Line Style Palette */
       case 102: /* Light Source Palette */
+      case 103: /* Reserved */
+      case 104: /* Reserved */
       case 112: /* Texture Mapping */
       case 114: /* Name Table */
 	 /* these are safe to ignore */
@@ -2484,6 +2589,7 @@ static int TableChunks(ubyte *ptr0, ubyte *end, fltState *state)
 	 break;
 
       default:
+	//printf("op %d: end of table chunks\n", op);
          done = 1;
       }
    }
@@ -2580,7 +2686,7 @@ static void ptree(ssgEntity *node, FILE *f, int d)
       for (i = 0; i <= lod->getNumKids(); i++)
 	 fprintf(f, " %.2f", lod->getRange(i));
    }
-   fputc('\n', f);
+   putc('\n', f);
    if (node->isAKindOf(ssgTypeBranch())) {
       ssgBranch *grp = (ssgBranch *)node;
       int n = grp->getNumKids();
@@ -2609,7 +2715,27 @@ static const char *FindFile(const char *file)
 
 static struct snode *FltCache;
 
-#define win32_perror(s) perror(s) /* no idea if this is appropriate.. */
+#ifdef UL_WIN32
+
+static void win32_perror(const char *s)
+{
+   LPVOID lpMsgBuf;
+   FormatMessage(
+        FORMAT_MESSAGE_ALLOCATE_BUFFER |
+        FORMAT_MESSAGE_FROM_SYSTEM |
+        FORMAT_MESSAGE_IGNORE_INSERTS,
+        NULL,
+        GetLastError(),
+        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+        (LPTSTR) &lpMsgBuf,
+        0,
+        NULL
+    );
+    fprintf(stderr, "%s: %s\n", s, lpMsgBuf);
+    LocalFree( lpMsgBuf );
+}
+
+#endif
 
 static ssgEntity *LoadFLT(const char *file)
 {
@@ -2649,7 +2775,7 @@ static ssgEntity *LoadFLT(const char *file)
 
 #ifdef USE_WIN32_MMAP
 	 
-	 fd = CreateFile(path, GENERIC_READ, 0, NULL, OPEN_EXISTING, 
+	 fd = CreateFile(path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 
 			 FILE_ATTRIBUTE_NORMAL, 0);
 	 if (fd == INVALID_HANDLE_VALUE) {
 	    win32_perror(path);
@@ -2704,7 +2830,7 @@ static ssgEntity *LoadFLT(const char *file)
 
 #endif /* ! USE_WIN32_MMAP */
 	 
-	 /*fputc('\n', stderr);*/
+	 /*putc('\n', stderr);*/
 
 	 name = strrchr(file, '/');
 	 if (name)
@@ -2813,11 +2939,7 @@ static ssgEntity *Flatten(ssgEntity *node, float (*mat)[4])
 	    ssgBranch *grp = (ssgBranch *)node;
 	    int i, n = grp->getNumKids();
 	    if (n > 0) {
-#ifdef USE_ALLOCA
-	       ssgEntity **kids = (ssgEntity **)alloca(sizeof(ssgEntity *) * n);
-#else
-	       ssgEntity *kids[n];
-#endif
+	       ssgEntity *kids[32];
 	       for (i = n; i--;) {
 		  kids[i] = grp->getKid(i);
 		  kids[i]->ref(); grp->removeKid(i); kids[i]->deRef();
@@ -2892,11 +3014,7 @@ static ssgEntity *Flatten(ssgEntity *node, float (*mat)[4])
 	 return Flatten(kid, mat);
       }
       else {
-#ifdef USE_ALLOCA
-	 ssgEntity **kids = (ssgEntity **)alloca(sizeof(ssgEntity *) * n);
-#else
-	 ssgEntity *kids[n];
-#endif
+	 ssgEntity *kids[32];
 	 for (i = n; i--;) {
 	    kids[i] = grp->getKid(i);
 	    kids[i]->ref(); grp->removeKid(i); kids[i]->deRef();
