@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgSelector.cxx,v 1.8 2001/06/16 16:27:37 stromberg Exp $
+     $Id: ssgSelector.cxx,v 1.12 2002/09/02 06:05:48 sjbaker Exp $
 */
 
 
@@ -95,7 +95,31 @@ void ssgSelector::hot ( sgVec3 sp, sgMat4 m, int test_needed )
 
   _ssgPopPath () ;
 
-  postTravTests ( SSGTRAV_HOT ) ; 
+  postTravTests ( SSGTRAV_HOT ) ;
+}
+
+
+void ssgSelector::los ( sgVec3 sp, sgMat4 m, int test_needed )
+{
+  if ( ! preTravTests ( &test_needed, SSGTRAV_LOS ) )
+    return ;
+
+  int los_result = los_test ( sp, m, test_needed ) ;
+
+  if ( los_result == SSG_OUTSIDE )
+    return ;
+
+  int s = 0 ;
+
+  _ssgPushPath ( this ) ;
+
+  for ( ssgEntity *e = getKid ( 0 ) ; e != NULL ; e = getNextKid(), s++ )
+    if ( selection [s] )
+      e -> los ( sp, m, los_result != SSG_INSIDE ) ;
+
+  _ssgPopPath () ;
+
+  postTravTests ( SSGTRAV_LOS ) ;
 }
 
 
@@ -126,6 +150,8 @@ void ssgSelector::isect ( sgSphere *sp, sgMat4 m, int test_needed )
 int ssgSelector::load ( FILE *fd )
 {
   _ssgReadInt ( fd, & max_kids ) ;
+  delete [] selection ;
+  selection = new unsigned char [ max_kids ] ;
   for ( int i=0; i<max_kids; i++ )
   {
     int temp ;
